/*  lircrc-config pretends to be a GUI for LIRC's rc files.
    Copyright (C) 2001 Manuel Estrada Sainz <ranty@atdot.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include <glade/glade.h>
#include <gnome.h>
#include "gtk-helpers.h"

void widget_changed(GtkWidget *widget)
{
	gtk_object_set_data(GTK_OBJECT(widget), "changed",
			    GINT_TO_POINTER(TRUE));
}

void widget_changed_apply(GtkWidget *widget, void *data)
{
	if (!is_widget_changed(widget))
		return;


	if (GTK_IS_SPIN_BUTTON(widget)){
		gint *val = (gint *)data;
		*val = gtk_spin_button_get_value_as_int(
			GTK_SPIN_BUTTON(widget));
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), 0);
		widget_changed_cancel(widget);
	} else if(GTK_IS_EDITABLE(widget)){
		gchar **string = (gchar **)data;
		g_free(*string);
		*string = gtk_editable_get_chars(GTK_EDITABLE(widget), 0, -1);
		gtk_editable_set_text(GTK_EDITABLE(widget), "", NULL);
		widget_changed_cancel(widget);
	}
}
void widget_changed_apply_recursive(GtkWidget *widget, void *data)
{
	gtk_container_foreach_recursive (widget,
					 (GtkCallback)widget_changed_apply,
					 data);
}
gboolean widget_changed_cancel(GtkWidget *widget)
{
	gboolean changed;

	changed = is_widget_changed(widget);

	gtk_object_set_data(GTK_OBJECT(widget), "changed", NULL);

	return changed;
}
gboolean is_widget_changed(GtkWidget *widget)
{
	gboolean changed;

	changed = GPOINTER_TO_INT(
		gtk_object_get_data(GTK_OBJECT(widget), "changed")); 
	return changed;
}


static void is_widget_changed_cb(GtkWidget *wid, gboolean *is_changed)
{
	*is_changed |= is_widget_changed(wid);
}
gboolean is_widget_changed_recursive(GtkWidget *widget)
{
	gboolean is_changed = FALSE;

	gtk_container_foreach_recursive (widget,
					 (GtkCallback)is_widget_changed_cb,
					 &is_changed);
	return is_changed;
}

void connect_changed_signal(GtkWidget *wid)
{
	gchar *signal_name;

	if(GTK_IS_EDITABLE(wid))
		signal_name="changed";
	else if(GTK_IS_TOGGLE_BUTTON(wid))
		signal_name="toggled";
	else
		g_assert(FALSE);

	gtk_signal_connect(GTK_OBJECT(wid),
			   signal_name,
			   widget_changed,
			   NULL);
}

void gtk_editable_set_text(GtkEditable *editable, gchar *text, GdkFont *font)
{
	gint pos=0;

	if(text == NULL)
		return;

	gtk_editable_delete_text (editable, 0, -1);
	if(!GTK_IS_TEXT(editable))
		gtk_editable_insert_text (editable, text, strlen(text), &pos);
	else
		gtk_text_insert (GTK_TEXT(editable), font,
				 NULL, NULL, text, -1);
	gtk_editable_set_position (editable, 0);

}

void gtk_container_foreach_recursive (GtkWidget * wid,
				      GtkCallback cb,
				      gpointer arg)
{
	static gint recursing = 0;
	static GtkCallback callback;
	static gpointer argument;

	if(recursing==0){
		callback = cb;
		argument = arg;
	}
	recursing++;

	if(GTK_IS_CONTAINER(wid))
		gtk_container_foreach(GTK_CONTAINER(wid),
				      (GtkCallback)
				      gtk_container_foreach_recursive,
				      NULL);
	callback(wid, argument);

	recursing--;
}
