#include "config.h"
#include <glade/glade.h>
#include <gnome.h>
#include <lirc/lirc_client.h>
#include "lircd_query.h"
#include "interface-lircd.h"
#include "gtk-helpers.h"

static GtkCombo *remote_combo;
static GtkCombo *button_combo;

static GtkEditable *remote_entry;
static GtkEditable *button_entry;

static GtkWindow *main_window;

static gint gdk_input_tag = -1;

static gboolean have_focus = FALSE;

#warning revise reconection with lircd and error reporting.

static void lircd_button_pressed (gpointer data,
				  gint source,
				  GdkInputCondition condition);

static void dialog_cb(gint reply, gpointer dummy)
{
	gint lircd;
	if(reply == 0) { /* YES */
		if((lircd=lirc_init(PACKAGE, 0))==-1){
			g_message("Couldn't connect to lircd\n");
			return;
		}
		gdk_input_tag = gdk_input_add(lircd, GDK_INPUT_READ,
					      lircd_button_pressed, NULL);
	}
}

static void lircd_button_pressed (gpointer data,
				  gint source,
				  GdkInputCondition condition)
{
	gchar *code;
	gchar *remote;
	gchar *button;

	if(lirc_nextcode(&code)==0 && (code!=NULL)){
		strtok(code, " \n"); /* skip the button code */
		strtok(NULL, " \n"); /* skip the button count */
		button = strtok(NULL, " \n");
		remote = strtok(NULL, " \n");

		if(have_focus){
			gtk_editable_set_text(remote_entry, remote, NULL);
			gtk_editable_set_text(button_entry, button, NULL);
		} else {
			g_print("We don't have the focus\n");
		}
		free(code);
	} else {
		GtkWidget *dialog;

		gdk_input_remove(gdk_input_tag);
		lirc_deinit();

		dialog = gnome_question_dialog_modal_parented(
			"Lost connection with lircd\n"
			"Should I try to restore it?",
			dialog_cb,
			NULL,
			main_window);
	}
}

static void remote_selected(GtkList *list,
			    GtkWidget *widget,
			    gpointer user_data)
{
	GSList *buttons = gtk_object_get_data(GTK_OBJECT(widget),
					      "button_list");
	gchar *current_button = gtk_editable_get_chars(button_entry, 0, -1);

  	gtk_combo_set_popdown_strings(button_combo, buttons);

	/* setting the popdown strings on the combo changes the
	   content of the entry, so we restore it */
	gtk_editable_set_text(button_entry,
	current_button, NULL); g_free(current_button);
}


static void add_remote(gchar *remote, gpointer arg)
{
	GSList *buttons;
	gint lircd = GPOINTER_TO_INT(arg);
	GtkList *remote_list = GTK_LIST(remote_combo->list);
	GtkWidget *item = gtk_list_item_new_with_label(remote);

	buttons = lirc_get_remote_buttons(lircd, remote);

	gtk_object_set_data(GTK_OBJECT(item), "button_list", buttons);

	gtk_container_add(GTK_CONTAINER(remote_list), item);
	gtk_widget_show (item);

}

gboolean event (GtkWidget *widget, GdkEvent *event,
		   gpointer user_data)
{
	if(event->type==GDK_FOCUS_CHANGE){
		GdkEventFocus *focus = (GdkEventFocus *)event;
		if(focus->in){
			g_print("Focus in\n");
			have_focus=TRUE;
		} else {
			g_print("Focus out\n");
			have_focus=FALSE;
		}
	}
}

void init_interface_lircd(GladeXML *xml)
{
	gint lircd;
	GSList *remotes;
	GtkList *remote_list;

	/* get required widget pointers */
	remote_combo = GTK_COMBO(glade_xml_get_widget(xml, "remote_combo"));
	button_combo = GTK_COMBO(glade_xml_get_widget(xml, "button_combo"));

	remote_entry = GTK_EDITABLE(glade_xml_get_widget(xml, "remote_entry"));
	button_entry = GTK_EDITABLE(glade_xml_get_widget(xml, "button_entry"));

	remote_list = GTK_LIST(remote_combo->list);

	main_window = GTK_WINDOW(glade_xml_get_widget(xml, "MainWindow"));

	gtk_signal_connect(GTK_OBJECT(main_window), "focus-in-event",
			   (GtkSignalFunc)event, NULL);
	gtk_signal_connect(GTK_OBJECT(main_window), "focus-out-event",
			   (GtkSignalFunc)event, NULL);

	if((lircd=lirc_init(PACKAGE, 0))==-1){
		g_message("Couldn't connect to lircd\n");
		return;
	}
	
	remotes = lirc_get_remotes(lircd);

	gtk_signal_connect(GTK_OBJECT(remote_list), "select-child",
			   remote_selected, NULL);

	g_slist_foreach(remotes, (GFunc)add_remote, GINT_TO_POINTER(lircd));

	widget_changed_cancel(GTK_WIDGET(remote_entry));
	widget_changed_cancel(GTK_WIDGET(button_entry));

	gdk_input_tag = gdk_input_add(lircd, GDK_INPUT_READ,
				      lircd_button_pressed, NULL);

}
