#include <gnome.h>
#include <glade/glade.h>
#include <lirc/lirc_client.h>
#include "parser.h"
#include "gtk-helpers.h"
#include "interface-lists.h"
enum {list=0, entry, add_button, rep_button, del_button, N_WIDGETS};

struct context{
	GtkWidget *wid[N_WIDGETS];
};

static GtkEntry *remote_entry;

static struct lirc_list **find_lirc_list(struct lirc_list **first,
					 struct lirc_list *item)
{
	g_assert(first);
	while (*first){
		if(*first == item)
			return first;
		first = &((*first)->next);
	}
	return first;
}
static struct lirc_code **find_lirc_code(struct lirc_code **first,
					 struct lirc_code *item)
{
	g_assert(first);
	while (*first){
		if(*first == item)
			return first;
		first = &((*first)->next);
	}
	return first;
}
static void add_button_clicked(GtkButton *button, struct context *ctx)
{
	GtkWidget *item;
	gchar *text = gtk_editable_get_chars(GTK_EDITABLE(ctx->wid[entry]),
					     0, -1);
	struct lirc_code **lirc_code_p;
	struct lirc_list **lirc_config_p;

	if(text[0] == '\0'){
		g_free(text);
		text = g_strdup("*");
	}

	item = gtk_list_item_new_with_label(text);
	gtk_widget_show (item);
	gtk_container_add(GTK_CONTAINER(ctx->wid[list]), item);
#if 0
	gtk_object_set_data(GTK_OBJECT(item), "entry_string",
			    &entry->next_code->button);
#endif
	lirc_code_p = gtk_object_get_data(GTK_OBJECT(ctx->wid[list]),
					  "lirc_code");
	lirc_config_p = gtk_object_get_data(GTK_OBJECT(ctx->wid[list]),
					    "lirc_config");
	if(lirc_code_p){
		struct lirc_code *new_code = g_new0(struct lirc_code, 1);
		new_comments(new_code, 1);
		lirc_code_p = find_lirc_code(lirc_code_p, NULL);
		*lirc_code_p = new_code;

		new_code->remote =
			gtk_editable_get_chars(GTK_EDITABLE(remote_entry),
					       0, -1);
		new_code->button = text;
		
	} else if (lirc_config_p){
		struct lirc_list *new_config = g_new0(struct lirc_list, 1);
		new_comments(new_config, 1);
		lirc_config_p = find_lirc_list(lirc_config_p, NULL);
		*lirc_config_p = new_config;

		new_config->string = text;
	}
}
static void rep_button_clicked(GtkButton *button, struct context *ctx)
{
	GtkWidget *label;
	struct lirc_code *lirc_code;
	struct lirc_list *lirc_config;
	GList *selection = GTK_LIST(ctx->wid[list])->selection;
	gchar *text;

	if(!selection)
		return;

	text = gtk_editable_get_chars(GTK_EDITABLE(ctx->wid[entry]), 0, -1);
	if(text[0] == '\0'){
		g_free(text);
		text = g_strdup("*");
	}

	label = GTK_BIN(selection->data)->child;
	gtk_label_set_text(GTK_LABEL(label), text);

	/* update lirc data structures */
	lirc_code = gtk_object_get_data(GTK_OBJECT(selection->data),
					"lirc_code");
	lirc_config = gtk_object_get_data(GTK_OBJECT(selection->data),
					  "lirc_config");
	if (lirc_code){
		g_free(lirc_code->button);
		g_free(lirc_code->remote);

		lirc_code->remote =
			gtk_editable_get_chars(GTK_EDITABLE(remote_entry),
					       0, -1);
		lirc_code->button = text;
	} else if(lirc_config){
		g_free(lirc_config->string);
		lirc_config->string = text;
	} else {
		g_free(text);
	}
}
static void del_button_clicked(GtkButton *button, struct context *ctx)
{
	GtkWidget *item;
	GList *selection = GTK_LIST(ctx->wid[list])->selection;
	gint last_pos;
	struct lirc_code *lirc_code;
	struct lirc_list *lirc_config;

	if(!selection)
		return;

	item = selection->data;
	last_pos = gtk_list_child_position (GTK_LIST(ctx->wid[list]), item);

	/* update lirc data structures */
	lirc_code = gtk_object_get_data(GTK_OBJECT(selection->data),
					"lirc_code");
	lirc_config = gtk_object_get_data(GTK_OBJECT(selection->data),
					  "lirc_config");
	if (lirc_code){
		struct lirc_code **lirc_code_p;

		lirc_code_p = gtk_object_get_data(GTK_OBJECT(ctx->wid[list]),
						  "lirc_code");
		lirc_code_p = find_lirc_code(lirc_code_p, lirc_code);
		*lirc_code_p = (*lirc_code_p)->next;

		g_free(lirc_code->button);
		g_free(lirc_code->remote);
		g_free(*get_comments(lirc_code));
		kill_comments(lirc_code);
		g_free(lirc_code);
	} else if(lirc_config){
		struct lirc_list **lirc_config_p;

		lirc_config_p = gtk_object_get_data(GTK_OBJECT(ctx->wid[list]),
						  "lirc_config");
		lirc_config_p = find_lirc_list(lirc_config_p, lirc_config);
		*lirc_config_p = (*lirc_config_p)->next;

		g_free(lirc_config->string);
		g_free(*get_comments(lirc_config));
		kill_comments(lirc_config);
		g_free(lirc_config);
	}

#warning free all dynamic memory in gtk_object_set_data
	gtk_widget_destroy(item);

	gtk_list_select_item (GTK_LIST(ctx->wid[list]), last_pos);

}

static void load_item(GtkListItem *item, struct context *ctx)
{
	struct lirc_code *lirc_code;
	struct lirc_list *lirc_config;

	lirc_code = gtk_object_get_data(GTK_OBJECT(item), "lirc_code");
	lirc_config = gtk_object_get_data(GTK_OBJECT(item), "lirc_config");

	if (lirc_code){
		gtk_editable_set_text(GTK_EDITABLE(remote_entry),
				      lirc_code->remote, NULL);
		gtk_editable_set_text(GTK_EDITABLE(ctx->wid[entry]),
				      lirc_code->button, NULL);
	} else if(lirc_config){
		gtk_editable_set_text(GTK_EDITABLE(ctx->wid[entry]),
				      lirc_config->string, NULL);
	}
}
static void list_selection_changed (GtkList *wid, struct context *ctx)
{
	gboolean selection = wid->selection != NULL;

	gtk_widget_set_sensitive(GTK_WIDGET(ctx->wid[rep_button]), selection);
	gtk_widget_set_sensitive(GTK_WIDGET(ctx->wid[del_button]), selection);
	if(selection)
		load_item(wid->selection->data, ctx);
}

void setup_interface_list (gchar *xml_prefix, GladeXML *xml)
{
	struct {
		gchar *name;
		gchar *signal;
		void *func;
	} wid[N_WIDGETS] = {
		{"list", "selection-changed", list_selection_changed},
		{"entry"},
		{"add_button", "clicked", add_button_clicked},
		{"rep_button", "clicked", rep_button_clicked},
		{"del_button", "clicked", del_button_clicked},
	};
	int i;
	GString *name = g_string_new(NULL);
	struct context *ctx = g_new0(struct context, 1);

	for(i=0; i< N_WIDGETS; i++){
		g_string_sprintf(name, "%s_%s", xml_prefix, wid[i].name);
		ctx->wid[i] = glade_xml_get_widget(xml, name->str);
		if(wid[i].signal)
			gtk_signal_connect_after(GTK_OBJECT(ctx->wid[i]),
					   wid[i].signal, wid[i].func, ctx);
	}
	remote_entry = GTK_ENTRY(glade_xml_get_widget(xml, "remote_entry"));

	/* make sure that rep/del buttons are properly sensitive */
	list_selection_changed (GTK_LIST(ctx->wid[list]), ctx);
}
