#include <errno.h>
#include <unistd.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>
#include <lirc/lirc_client.h>
#include <glib.h>
#include "lircd_query.h"

static GSList *lirc_get_reply(int lircd)
{
	char *code;
	GSList *reply=NULL;

	while(lirc_nextcode(&code)==0)
	{
		if(code==NULL) continue;
		if(code[strlen(code)-1] == '\n')
			code[strlen(code)-1]='\0';
		reply=g_slist_append(reply, code);
		if(strncmp(code, "END", 3)==0){
			break;
		}
	}
	return reply;
}
static GSList *lirc_send_command(int lircd, gchar *command)
{
	int len = strlen(command);
	if(write(lircd, command, len)!= len){
		perror("write");
		return NULL;
	}
	return lirc_get_reply(lircd);
}
static GSList *lirc_parse_reply(GSList *reply)
{
	GSList *data_lines=NULL;
	gint n, i;

	if(reply == NULL)
		return NULL;

	if(strcmp(reply->data, "BEGIN")!=0)
		return NULL;
	reply=reply->next;
	reply=reply->next; /* skip the command */
	if(strcmp(reply->data, "SUCCESS")!=0)
		return NULL;
	reply=reply->next;
	if(strcmp(reply->data, "DATA")!=0)
		return NULL;
	reply=reply->next;
	n = atoi(reply->data);
	reply=reply->next;
	for(i=0; i<n; i++){
		data_lines=g_slist_append(data_lines, reply->data);
		reply=reply->next;
	}
	if(strcmp(reply->data, "END")!=0)
		return NULL;

	return data_lines;
}
GSList *lirc_get_remotes(int lircd)
{
	GSList *reply;
	GSList *remotes=NULL;

	reply = lirc_send_command(lircd, "LIST\n");

	remotes=lirc_parse_reply(reply);
	g_slist_free(reply);

	return remotes;
}
GSList *lirc_get_remote_buttons_full(int lircd, gchar *remote)
{
	GSList *reply;
	GSList *buttons;
	GString *command = g_string_new("LIST");

	g_string_sprintfa(command, " %s\n", remote);
	reply = lirc_send_command(lircd, command->str);

	buttons = lirc_parse_reply(reply);
	g_slist_free(reply);

	return buttons;
}
static void cleanup_button_names(gchar *button, GSList **new_list)
{
	gchar *fist_space = strchr(button, ' ');
	*new_list = g_slist_append(*new_list, strdup(fist_space+1));
	free(button);
}
GSList *lirc_get_remote_buttons(int lircd, gchar *remote)
{
	GSList *buttons = lirc_get_remote_buttons_full(lircd, remote);
	GSList *clean_buttons = NULL;

	g_slist_foreach(buttons, (GFunc)cleanup_button_names, &clean_buttons);
	return clean_buttons;
}

