/*  lircrc-config pretends to be a GUI for LIRC's rc files.
    Copyright (C) 2001 Manuel Estrada Sainz <ranty@atdot.org>

    Initialy based on: test-libglade.c: a test program for the libglade library
    Copyright (C) 1998, 1999, 2000  James Henstridge <james@daa.com.au>
      GNOME option parsing code by Miguel de Icaza.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <errno.h>
#include <unistd.h>
#include <string.h>
#include <gnome.h>
#include <glade/glade.h>
#include "widget-groups.h"
#include "parser.h"
#include "comments.h"
#include "interface.h"
#include "file-dialog.h"

#define GLADE_XML_FILE "lircrc-config.glade"
gboolean no_connect = FALSE;

static poptContext ctx;

static const struct poptOption options[] = {
	{"no-connect", '\0', POPT_ARG_INT, &no_connect, 0,
	 N_("Do not connect signals")},
	{NULL, '\0', 0, NULL, 0}
};

int main(int argc, char **argv)
{
	GladeXML *xml;
	const char **list = NULL;
	
	gnome_init_with_popt_table(PACKAGE, VERSION, argc, argv,
				   options, 0, &ctx);
	glade_gnome_init();

	if(access(GLADE_XML_FILE, R_OK)==0)
		/* We may be running from the source tarball, read the newest
		 * development version of the interface */
		xml = glade_xml_new(GLADE_XML_FILE, NULL);
	else if(access(PKGDATADIR "/" GLADE_XML_FILE, R_OK)==0)
		/* Use the installed version otherwise */
		xml = glade_xml_new(PKGDATADIR "/" GLADE_XML_FILE, NULL);
	else 
		g_error("Couldn't load the interface file: %s",
			strerror(errno));

	if (xml == NULL)
		g_error("something bad happened while creating the interface");

	widget_group_setup(xml);

	glade_xml_signal_connect(xml, "gtk_main_quit", gtk_main_quit);

	init_comments();
	init_interface(xml);

	//gtk_object_unref(GTK_OBJECT(xml));

	list = poptGetArgs(ctx);
	if(list!=NULL)
		open_config_file(list[0]);
	else if(getenv("HOME")){
		GString *str = g_string_new(NULL);
		g_string_sprintf(str, "%s/.lircrc", getenv("HOME"));
		open_config_file(str->str);
		g_string_free(str, TRUE);
	}

	gtk_main();

	return 0;
}
