/*  lircrc-config pretends to be a GUI for LIRC's rc files.
    Copyright (C) 2001 Manuel Estrada Sainz <ranty@atdot.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
%option nounput
%option yylineno

%{
#include <glib.h>
#include <sys/types.h>
#include <string.h>
#include "parser.tab.h"
GString *comments = NULL;
static char *arrange_comments(GString **str)
{
	char *c_str = NULL;

	if(*str != NULL){
		c_str= (*str)->str;
		g_string_free(*str, FALSE);
		*str = NULL;
	} 
	return c_str;
}
%}


DIGIT		[0-9]
ID		[^ \t\n]+
RVAL_STRING	=[^\n]*
QUOT_STRING	\"[^"]*\"	
HEX_DIGIT	[0-9a-fA-F]
HEX		0x{HEX_DIGIT}+
DEC		{DIGIT}+
NEW_LINE	\n
EMPTY_LINE	^[ \t]*\n
COMMENT		#.*\n

%%
{EMPTY_LINE}	;
{COMMENT}	{
			if(comments == NULL)
				comments = g_string_new(yytext);
			else {
				g_string_sprintfa(comments, yytext);
			}
		}
{DEC}		yylval.num = atoi(yytext); return DEC;
{HEX}		yylval.num = strtol(yytext, NULL, 16); return HEX;
begin		yylval.str = arrange_comments(&comments); return BeGIN;
prog		yylval.str = arrange_comments(&comments); return PROG;
remote		yylval.str = arrange_comments(&comments); return REMOTE;
button		yylval.str = arrange_comments(&comments); return BUTTON;
repeat		yylval.str = arrange_comments(&comments); return REPEAT;
config		yylval.str = arrange_comments(&comments); return CONFIG;
mode		yylval.str = arrange_comments(&comments); return MODE;
flags		yylval.str = arrange_comments(&comments); return FLAGS;
end		yylval.str = arrange_comments(&comments); return END;

{QUOT_STRING}	{
		yytext[yyleng-1]='\0';
		yylval.str = yytext+1;
		return QUOT_STRING;
		}
{RVAL_STRING}	{
		char *text = yytext;
		for(text++; *text==' '; text++){};
		yylval.str = g_strdup(text);
		return RVAL_STRING;
		}
=		return yytext[0];
{ID}		{
		yylval.str = g_strdup(yytext);
		return ID;
		}
" "|\t		
{NEW_LINE}	return yytext[0];
.		{
		printf("lex error in line %d \"%s\"\n", yylineno, yytext);
		return 0;
		}
